% This program takes the total sample size of parts to be tested, the
% desired confidence limit, and the maximum number of part failures
% allowable in that sample size. It can then use this information to
% Approximate the upper and lower confidence limits for the Binomial and
% Poisson distributions. For small sample sizes, a Binomial distributions
% is used. When sample sizes are greater than 100, a Poisson distribution
% is used to approximate the upper and lower confidence limits which
% coincide with consumer and producer risks associated with different
% sampling schedules.

% user inputs based on desired sample size (num_samples), confidence limit
% (confidence_lim), and number of observed failures in the sample
% (num_of_failures)
num_samples = 10;
confidence_lim = 0.9;
num_of_failures = 0;

% do not alter the code below here

n = num_samples;
a = num_of_failures;

% only return positive and real solutions of p
syms p positive

%create equation to solve: All of the equations will include the
%probability that zero failures will occur as seen in the binomial 
%distribution formula below where a=0
eqn = nchoosek(n, 0)*(p)^(0)*(1-p)^(n-0);

%For the Upper limit, the desired value of the binomial distribution is
%1-desired_confidence_limit. For the lower limit, it will be equal to the
%desired_confidence_limit.
Binomial_dist = 1-confidence_lim;

%If the number of failures found equals zero, solve for the upper limit
%immediately by solving for p, the probability that a sample fails.
if a==0
    %Solve for p
    k = [solve(Binomial_dist == eqn, p)];
    %Approximate p as a rounded number
    k = vpa(k);
    %Calculate the Upper confidence limit based on the number of samples,
    %n, being tested.
    Upper_limit = min(k)*n
    
    %For the lower confidence limit, the binomial distribution will be
    %equal to the desired_confidence_limit.
    Binomial_dist = confidence_lim;
    %Solve for p
    k = [solve(Binomial_dist == eqn, p)];
    %Approximate p as a rounded number
    k = vpa(k);
    %Calculate the Lower confidence limit based on the number of samples,
    %n, being tested.
    Lower_limit = min(k)*n
    
%If the number of failures that occur is more than zero, we must add each
%binomial distribution to the previous. For example, if at least two parts 
%fail, we need to include the probabilities that one part fails and that no
%parts fail. Then, as above, solve the equation for the probability, p,
%that at least the number of parts stated fail
elseif n<100
    for m = 1:(a)
        %Create equation to solve:
        eqn = eqn + nchoosek(n, m).*(p).^m.*(1-p).^(n-m);
    end
    % solve for the upper and lower confidence limits based on the number
    % of samples being tested, n.
    Binomial_dist = 1-confidence_lim;
    k = [solve(Binomial_dist == eqn, p)];
    k = vpa(k);
    Upper_limit = min(k)*n
    Binomial_dist = confidence_lim;
    k = [solve(Binomial_dist == eqn, p)];
    k = vpa(k);
    Lower_limit = min(k)*n

%If the number of samples is large, the approximate the value confidence
%limits using the Poisson distribution. Repeat the same solving process for
%the probability of failures detected, p, as above.
elseif n>=100
    eqn = (exp(1)^(-n*p)*(n*p)^0)/factorial(0);
    if a>0
        for m = 1:(a)
            eqn = eqn + (exp(1)^(-n*p)*(n*p)^m)/factorial(m);
        end
    end
    Poisson_dist = 1-confidence_lim;

    %Need to initiate different guesses for value of p to reach convergence
    %when solving for p.
    for c=0:0.01:1
        k = vpasolve(0.1==eqn,p,c);
        if k>0
            break
        end
    end
    Upper_limit = min(k)*n
    Poisson_dist = confidence_lim;
    k = [vpasolve(Poisson_dist == eqn, p)];
    Lower_limit = min(k)*n
    
end



    
        